float4x4 World;
float4x4 View;
float4x4 Projection;
float4 LightPosition;
float4 EyePosition;
float3 Attenuation012;

float4 AmbientLight;
float4 DiffuseLight;
float4 SpecularLight;

float4 AmbientMaterial;
float4 DiffuseMaterial;
float4 SpecularMaterial;

struct VertexShaderInput
{
    float4 Position : POSITION;
    float3 Normal : NORMAL;
};

struct VertexShaderOutput
{
    float4 Position : POSITION;	// transformed position
    float3 L : TEXCOORD0;		// light
    float3 N : TEXCOORD1;		// surface normal
    float3 V : TEXCOORD2;		// view
    float A : TEXCOORD3;		// attenuation
};

VertexShaderOutput VertexShaderFunction(VertexShaderInput input)
{
    VertexShaderOutput output = (VertexShaderOutput) 0;
    
    float4 worldPos = mul(input.Position, World);
    float4 viewPos = mul(worldPos, View);
    output.Position = mul(viewPos, Projection);	
    
   	// transform a surface normal from model space to world space
    output.N = mul(input.Normal, World);				
    
    // light direction 
    output.L = LightPosition;

    // view direction (Eye position - vertex position) from eye to the vertex.
    output.V = EyePosition - worldPos;

	// light attenuation
	float d = distance(LightPosition.xyz, worldPos.xyz);
	output.A = Attenuation012.x + Attenuation012.y * d + Attenuation012.z * d * d;

	return output;
}

float4 PixelShaderFunction(VertexShaderOutput input) : COLOR
{
    // normalize our vectors.
    float3 Normal = normalize(input.N);
    float3 LightDir = normalize(input.L);
    float3 ViewDir = normalize(input.V);    
    
    // calculate ambient
    float4 Ambient = AmbientLight * AmbientMaterial;
    
    // calculate diffuse (N.L)
    float s = saturate(dot(Normal, LightDir));
    float4 Diffuse = s * DiffuseLight * DiffuseMaterial;
    
    // calculate specular
    // create our reflection shader R = 2 * (N.L) * N - L
    float3 Reflect = normalize(2 * s * Normal - LightDir);  
    float t = pow(saturate(dot(Reflect, ViewDir)), 20); // R.V^n
    float4 Specular = t * SpecularLight * SpecularMaterial;
	
    // return our final light equation    
    //return Ambient + Diffuse + Specular;
    return Ambient + ((Diffuse + Specular)/input.A);
}

technique AmbientDiffuseSpecularLightTechnique
{
    pass Pass1
    {
        VertexShader = compile vs_2_0 VertexShaderFunction();
        PixelShader = compile ps_2_0 PixelShaderFunction();	// need to have ps_2_0
    }
}
