﻿//-----------------------------------------------------------------------------
// Game1.cs
//
// Kyoung Shin Park @ Dankook University (4/4/2011)
//-----------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using XNAGeometryObject;

namespace XNAGeometryEffect
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // effect
        Effect effect;
        Matrix projection;
        Matrix view;
        Matrix world;

        // input
        GamePadState currentGamePadState;
        GamePadState prevGamePadState;
        KeyboardState currentKeyboardState;
        KeyboardState prevKeyboardState;

        // Store a list of primitive models, plus which one is currently selected.
        List<GeometricPrimitive> primitives = new List<GeometricPrimitive>();
        float angle = 0.0f;
        float zoom = 1.0f;
        bool isWireframe = true;
        bool isAnimation = true;

        // ambient, diffuse, specular light color
        Vector4 ambientLight = new Vector4(0.2f, 0.2f, 0.2f, 1.0f);
        Vector4 diffuseLight = new Vector4(1.0f, 1.0f, 1.0f, 1.0f);
        Vector4 specularLight = new Vector4(1.0f, 1.0f, 0.0f, 1.0f);

        Vector4 ambientMaterial;
        Vector4 diffuseMaterial;
        Vector4 specularMaterial;
        
        Vector4 lightPosition = new Vector4(3.0f, 5.0f, 0.0f, 1.0f);
        Vector4 eyePosition = new Vector4(0.0f, 0.0f, 5.0f, 0.0f);  // same as LookAt
        Vector3 lightAttenuation = new Vector3(1.0f, 0.0f, 0.0f);

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            InitializeTransform();
            InitializeEffect();

            base.Initialize();
        }

        /// <summary>
        /// Initializes the transforms used by the game.
        /// </summary>
        private void InitializeTransform()
        {
            // world matrix
            world = Matrix.Identity;

            // view matrix
            Vector3 cameraPosition = new Vector3(0, 0, 5.0f);
            view = Matrix.CreateLookAt(cameraPosition, Vector3.Zero, Vector3.Up);

            // projection matrix
            float aspect = GraphicsDevice.Viewport.AspectRatio;
            projection = Matrix.CreatePerspectiveFieldOfView(1, aspect, 1, 1000);
        }

        /// <summary>
        /// Initializes the effect (loading, parameter setting, and technique selection)
        /// used by the game.
        /// </summary>
        private void InitializeEffect()
        {
            // Initialize Effect
            effect = Content.Load<Effect>("AmbientDiffuseSpecularLightMaterial");

            effect.Parameters["World"].SetValue(world);
            effect.Parameters["View"].SetValue(view);
            effect.Parameters["Projection"].SetValue(projection);

            effect.Parameters["AmbientLight"].SetValue(ambientLight);
            effect.Parameters["DiffuseLight"].SetValue(diffuseLight);
            effect.Parameters["SpecularLight"].SetValue(specularLight);
            effect.Parameters["Attenuation012"].SetValue(lightAttenuation);
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
            primitives.Add(new CubePrimitive(GraphicsDevice));
            primitives.Add(new SpherePrimitive(GraphicsDevice));
            primitives.Add(new CylinderPrimitive(GraphicsDevice));
            primitives.Add(new TorusPrimitive(GraphicsDevice));
            //primitives.Add(new TeapotPrimitive(GraphicsDevice));
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            HandleInput();

            base.Update(gameTime);
        }

        #region HandleInput
        protected void HandleInput()
        {
            prevGamePadState = currentGamePadState;
            currentGamePadState = GamePad.GetState(PlayerIndex.One);
            prevKeyboardState = currentKeyboardState;
            currentKeyboardState = Keyboard.GetState(PlayerIndex.One);

            // Check for exit.
            if (IsPressed(Keys.Escape, Buttons.Back))
            {
                Exit();
            }

            // Toggle animation?
            if (IsPressed(Keys.A, Buttons.A))
            {
                isAnimation = !isAnimation;
            }

            // Toggle wireframe?
            if (IsPressed(Keys.W, Buttons.B))
            {
                isWireframe = !isWireframe;
            }

            // Change Y angle?
            if (IsPressed(Keys.X, Buttons.X))
            {
                angle += 5.0f;
            }

            // Change Translate?
            if (IsPressed(Keys.Y, Buttons.Y))
            {
                zoom -= 0.1f;
            }

            // Change LightPosition?
            if (IsPressed(Keys.L, Buttons.DPadLeft))
            {
                lightPosition.X -= 0.5f;
            }

            if (IsPressed(Keys.R, Buttons.DPadRight))
            {
                lightPosition.X += 0.5f;
            }
        }

        /// <summary>
        /// Checks whether the specified key or button has been pressed.
        /// </summary>
        bool IsPressed(Keys key, Buttons button)
        {
            return (currentKeyboardState.IsKeyDown(key) &&
                    prevKeyboardState.IsKeyUp(key)) ||
                   (currentGamePadState.IsButtonDown(button) &&
                    prevGamePadState.IsButtonUp(button));
        }
        #endregion

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
            RenderState renderState = GraphicsDevice.RenderState;
            if (isWireframe)
            {
                renderState.FillMode = FillMode.WireFrame;
                renderState.CullMode = CullMode.None;
            }
            else
            {
                renderState.FillMode = FillMode.Solid;
                renderState.CullMode = CullMode.CullCounterClockwiseFace;
            }

            // Create camera matrices, making the object spin.
            float time = (float)gameTime.TotalGameTime.TotalSeconds;
            if (isAnimation) angle = time * 30.0f;

            effect.Parameters["LightPosition"].SetValue(lightPosition);
            effect.Parameters["EyePosition"].SetValue(eyePosition);

            // Draw the transformed cube, sphere, cylinder, torus primitives.
            world = Matrix.CreateRotationY(MathHelper.ToRadians(angle)) * Matrix.CreateTranslation(0, 0, zoom) * Matrix.CreateTranslation(-1.0f, 0.0f, 0.0f);
            effect.Parameters["World"].SetValue(world);
            ambientMaterial = Color.Blue.ToVector4();
            diffuseMaterial = Color.Blue.ToVector4();
            specularMaterial = Color.Yellow.ToVector4();
            effect.Parameters["AmbientMaterial"].SetValue(ambientMaterial);
            effect.Parameters["DiffuseMaterial"].SetValue(diffuseMaterial);
            effect.Parameters["SpecularMaterial"].SetValue(specularMaterial);
            primitives[0].Draw(effect);

            world = Matrix.CreateRotationY(MathHelper.ToRadians(angle)) * Matrix.CreateTranslation(0, 0, zoom) * Matrix.CreateScale(1.5f, 1.5f, 1.5f) * Matrix.CreateTranslation(1.0f, 0.0f, 0.0f);
            effect.Parameters["World"].SetValue(world);
            ambientMaterial = Color.Red.ToVector4();
            diffuseMaterial = Color.Red.ToVector4();
            specularMaterial = Color.Yellow.ToVector4();
            effect.Parameters["AmbientMaterial"].SetValue(ambientMaterial);
            effect.Parameters["DiffuseMaterial"].SetValue(diffuseMaterial);
            effect.Parameters["SpecularMaterial"].SetValue(specularMaterial);
            primitives[1].Draw(effect);

            world = Matrix.CreateRotationX(MathHelper.ToRadians(angle)) * Matrix.CreateTranslation(0, 0, zoom) * Matrix.CreateTranslation(0.0f, 0.0f, -1.0f);
            effect.Parameters["World"].SetValue(world);
            ambientMaterial = Color.Green.ToVector4();
            diffuseMaterial = Color.Green.ToVector4();
            specularMaterial = Color.Yellow.ToVector4();
            effect.Parameters["AmbientMaterial"].SetValue(ambientMaterial);
            effect.Parameters["DiffuseMaterial"].SetValue(diffuseMaterial);
            effect.Parameters["SpecularMaterial"].SetValue(specularMaterial);
            primitives[2].Draw(effect);

            world = Matrix.CreateRotationX(MathHelper.ToRadians(angle)) * Matrix.CreateTranslation(0, 0, zoom) * Matrix.CreateTranslation(0.0f, 0.0f, 1.0f);
            effect.Parameters["World"].SetValue(world);
            ambientMaterial = Color.Magenta.ToVector4();
            diffuseMaterial = Color.Magenta.ToVector4();
            specularMaterial = Color.Yellow.ToVector4();
            effect.Parameters["AmbientMaterial"].SetValue(ambientMaterial);
            effect.Parameters["DiffuseMaterial"].SetValue(diffuseMaterial);
            effect.Parameters["SpecularMaterial"].SetValue(specularMaterial);
            primitives[3].Draw(effect);

            base.Draw(gameTime);
        }
    }
}
